//
// Created by R on 9/25/23.
//

#include <cmath>
#include "AutoOffsetAdjustSM.h"

AutoOffsetAdjustSM::auto_offset_result_t AutoOffsetAdjustSM::step(const decay_sample_t &sample, float &new_dc) {
    sampleCounter++;
    if (sampleCounter > (1000000 / _sampling_cfg.pulse_period_us)) {
        float sum = 0.0f;
        uint32_t count;
        //use only the second half of samples for auto offset adjustment, as the first N samples could be zero, skewing the mean
        uint32_t offset = _sampling_cfg.num_samples_analyze/2;
        for (count = 0; count < _sampling_cfg.num_samples_analyze - offset; count++) {
            sum += sample.sample[count + offset];
        }

        assert(count > 0);

        auto mean = (uint32_t)(sum / count);
        if (mean > _sampling_cfg.auto_offset_adjustment_range_min && mean < _sampling_cfg.auto_offset_adjustment_range_max) {
            result = FINISHED;
        } else {
            if (waitSamplesToStabilize == 0) {
                currentOffset += AUTO_OFFSET_DUTY_CYCLE_STEP;
                waitSamplesToStabilize = WAIT_SAMPLES_STABILIZE;
            } else {
                waitSamplesToStabilize--;
            }
        }

        if (currentOffset > 1.0f) {
            currentOffset = 0.0f;
            numFailedCycles++;
            if (numFailedCycles > 2)
                result = FAILED;
        }
    }

    new_dc = currentOffset;
    return result;
}

void AutoOffsetAdjustSM::init() {
    sampleCounter = 0;
    currentOffset = 0.0f;
    waitSamplesToStabilize = WAIT_SAMPLES_STABILIZE;
    numFailedCycles = 0;
    result = IN_PROGRESS;
}

void AutoOffsetAdjustSM::setSamplingConfiguration(const SamplingConfiguration &sc) {
    _sampling_cfg = sc;
}
