//
// Created by R on 9/25/23.
//

#ifndef PIFW_AUTOOFFSETADJUSTSM_H
#define PIFW_AUTOOFFSETADJUSTSM_H

#include <stdint.h>
#include "structures.h"
#include "pi.pb.h"

class AutoOffsetAdjustSM {
public:
    explicit AutoOffsetAdjustSM(const SamplingConfiguration& sc): _sampling_cfg(sc) {
        init();
    }

    typedef enum {
        IN_PROGRESS=0,
        FINISHED=1,
        FAILED=2
    } auto_offset_result_t;

    /**
     * Performs a single step of auto offset adjustment and updates the new duty cycle to use
     * Returns true if
     */
    auto_offset_result_t step(const decay_sample_t& sample, float& new_dc);

    /**
     * Re-initializes state machine
     */
    void init();

    /**
     * How many samples to wait after updating duty cycle
     */
    static const uint32_t WAIT_SAMPLES_STABILIZE = 64;
    constexpr static const float AUTO_OFFSET_DUTY_CYCLE_STEP = 0.002f;

    void setSamplingConfiguration(const SamplingConfiguration& sc);
protected:
    SamplingConfiguration _sampling_cfg;
    auto_offset_result_t         result;
    float currentOffset{};
    uint32_t sampleCounter{};
    uint32_t waitSamplesToStabilize{};
    uint32_t numFailedCycles{};

};


#endif //PIFW_AUTOOFFSETADJUSTSM_H
