
#include <hardware/adc.h>
#include <algorithm>
#include <cmath>
#include "BatteryMonitor.h"
#include "util/math.h"

BatteryMonitor::BatteryMonitor() : _filtered_adc_value(0), _simulated_voltage(0.0f), _have_simulated_voltage(false) {
}

bool BatteryMonitor::init() {
    adc_init();
    adc_gpio_init(PIN_BATT_VOLTAGE);
    adc_select_input(BATT_VOLTAGE_ADC_NUM);
    _filtered_adc_value = adc_read();
    return true;
}

/**
 * @return Charge value 0-100 percentage
 */
BatteryMonitor::battery_charge_t BatteryMonitor::read_battery_charge() {
    battery_charge_t ret;
    ret.adc_counts = _filtered_adc_value;
    if (_have_simulated_voltage) {
        ret.battery_voltage = _simulated_voltage;
    } else {
        float pin_volt = _filtered_adc_value * 1.0f / ADC_MAX_VALUE * ADC_REFERENCE_VOLTAGE;
        ret.battery_voltage = pin_volt * ((ADC_VOLTAGE_DIVIDER_R1 + ADC_VOLTAGE_DIVIDER_R2) / ADC_VOLTAGE_DIVIDER_R2);
    }
    float soc = roundf(map<float>(ret.battery_voltage, BATTERY_VOLTAGE_MIN, BATTERY_VOLTAGE_MAX, 0, 100));
    ret.battery_soc = (uint8_t) std::clamp((int)soc, 0, 100);
    return ret;
}

void BatteryMonitor::update_battery_charge() {
    adc_select_input(BATT_VOLTAGE_ADC_NUM);
    const float beta = 0.95f;
    _filtered_adc_value = (1.0f - beta) * (float)adc_read() + beta * _filtered_adc_value;
}

void BatteryMonitor::set_simulated_battery_voltage(float volt) {
    _simulated_voltage = volt;
    _have_simulated_voltage = true;
}

void BatteryMonitor::clear_simulated_battery_voltage() {
    _have_simulated_voltage = false;
}

