#ifndef PIFW_BATTERYMONITOR_H
#define PIFW_BATTERYMONITOR_H

#include <sys/types.h>
#include <cstdint>

class BatteryMonitor {
public:
    BatteryMonitor();

    bool init();

    typedef struct {
        uint16_t adc_counts;
        float    battery_voltage;
        uint8_t  battery_soc;
    } battery_charge_t;

    void             update_battery_charge();
    battery_charge_t read_battery_charge();
    void set_simulated_battery_voltage(float volt);
    void clear_simulated_battery_voltage();
protected:
    uint16_t    _filtered_adc_value;
    static const uint32_t PIN_BATT_VOLTAGE = 29;
    static const uint32_t BATT_VOLTAGE_ADC_NUM = PIN_BATT_VOLTAGE - 26;

    static const uint16_t ADC_MAX_VALUE = 4095;
    constexpr static const float ADC_REFERENCE_VOLTAGE = 3.3f;
    constexpr static const float ADC_VOLTAGE_DIVIDER_R1 = 10.0f; // upper voltage divider resistor value in kOhm
    constexpr static const float ADC_VOLTAGE_DIVIDER_R2 = 20.0f; // lower resistor value in kOhm

    // Min/max battery voltage
    constexpr static const float BATTERY_VOLTAGE_MIN = 3.2f;
    constexpr static const float BATTERY_VOLTAGE_MAX = 4.2f;

    bool _have_simulated_voltage;
    float _simulated_voltage;
};

#endif //PIFW_BATTERYMONITOR_H
