//
// Created by R on 9/25/23.
//

#ifndef PIFW_PIMD_H
#define PIFW_PIMD_H

#include "sampler/PISampler.h"
#include "ProtocolComm.h"
#include "structures.h"
#include "AutoOffsetAdjustSM.h"
#include "RefSample.h"
#include "UI.h"
#include "PersistentParameters.h"
#include "IndicationStateMachine.h"
#include "ThresholdDetectorSM.h"
#include "BatteryMonitor.h"
#include "TemperatureMonitor.h"
#include "Vibro.h"

class PIMD {
public:
    PIMD(PISampler &pisampler, ProtocolComm &pc, UI &ui, IndicationStateMachine &ism,
         ThresholdDetectorSM &td, HAL &hal);

    typedef enum {
        STATE_INIT = 0,
        STATE_INITIALIZATION_AUDIO_PLAYBACK,
        STATE_START_AUTO_OFFSET_ADJUSTMENT,
        STATE_AUTO_OFFSET_ADJUSTMENT,
        STATE_START_REF_SAMPLE_ACQUISITION,
        STATE_REF_SAMPLE_ACQUISITION,
        STATE_THRESHOLD_INITIALIZATION,
        STATE_INDICATE_BATTERY_LEVEL,
        STATE_OPERATIONAL,
        STATE_ERROR_AUTO_OFFSET_FAILED,
        STATE_ERROR_REF_SAMPLE_ACQUISITION_FAILED
    } op_state_t;

    void step();

protected:
    op_state_t _op_state;

    PISampler&      _pi;
    ProtocolComm&   _pc;
    UI&             _ui;
    PersistentParameters& _params;
    IndicationStateMachine& _ism;
    ThresholdDetectorSM& _td;
    BatteryMonitor &_bm;
    Vibro& _vibro;
    TemperatureMonitor& _tm;
    PWMAudio& _audio;

    void state_step_init();
    void state_step_offset_adjustment();
    void state_step_ref_sample_acquisition();
    void state_step_threshold_initialization();
    void state_step_indicate_battery_level();
    void state_step_operational();

    void state_transition(op_state_t new_state);

    void process_protocol_commands();
    void process_user_commands();
    void periodic_send_reference_measurement();
    void periodic_battery_monitor_measurement();
    void periodic_send_system_status();

    AutoOffsetAdjustSM _auto_offset_sm;
    RefSample          _ref_sample;
    uint32_t           _numOperationalTransitions;
    uint64_t           _nextRefSampleConsumptionUs;
    void applyUISettings();
    SystemStatus       _statusStruct;
    uint64_t           _nextSystemStatusSendTimeUs;
    static const uint32_t SYSTEM_STATUS_TRANSMIT_PERIOD_US = 1 * 1000 * 1000;
    void update_system_status();
    struct {
        uint32_t numSamplesProcessed;
        uint64_t samplerateStartTimeUs;
    } _samplerateCounter;
    bool        _indicate_battery_on_startup;
    op_state_t  _state_after_battery_indication;
    static PWMAudio::system_sound_key_t battery_soc_to_level(uint8_t soc);
    struct {
        PWMAudio::system_sound_key_t prev_indicated_level;
        uint32_t num_indicated_low;
        uint32_t num_indicated_critical;
        uint64_t next_crit_indication_ts;
    } _battery_level_events;

    void periodic_temperature_measurement();

    uint64_t _offset_adjustment_end_timestamp_us;
    uint32_t _top_clipping_counter;
};


#endif //PIFW_PIMD_H
