//
// Created by R on 7/13/23.
//

#include "PersistentParameters.h"
#include <cstring>
#include <pico/multicore.h>

PersistentParameters::PersistentParameters() : wantsFlashWrite(false), canFlashWrite(false) {
    parameters.magic = 0;
    parameters.ui_volume = 1;
    parameters.ui_vibro_power = 1;
    parameters.ui_sensitivity = 10;
    parameters.ui_backlight_enabled = true;
    parameters.ui_brightness = 5;
    parameters.ui_detection_mode = DETECTION_MODE_STATIC;

    critical_section_init(&criticalSection);
}

bool PersistentParameters::readParameters() {
    memcpy(_pageBuffer, (uint8_t*)XIP_BASE + FLASH_TARGET_OFFSET, sizeof(_pageBuffer));
    uint32_t buf_magic = ((uint32_t*)_pageBuffer)[0];
    uint32_t size = ((uint32_t*)_pageBuffer)[1];
    if (buf_magic == EXPECTED_MAGIC && size == EXPECTED_SIZE) {
        memcpy(&parameters, _pageBuffer, sizeof(parameters));
        return true;
    }
    return false;
}

static void __no_inline_not_in_flash_func(write_parameters)(uint32_t target_offs, uint8_t* data_buffer, uint32_t buffer_size) {
    auto status = save_and_disable_interrupts();
    multicore_lockout_start_blocking();
    flash_range_erase(target_offs, FLASH_SECTOR_SIZE);
    flash_range_program(target_offs, data_buffer, buffer_size);
    multicore_lockout_end_blocking();
    restore_interrupts(status);
}

void PersistentParameters::writeParameters() {
    parameters.magic = EXPECTED_MAGIC;
    parameters.size = EXPECTED_SIZE;
    memset(_pageBuffer, 0, sizeof(_pageBuffer));
    memcpy(_pageBuffer, &parameters, sizeof(parameters));
    write_parameters(FLASH_TARGET_OFFSET, _pageBuffer, sizeof(_pageBuffer));
}

void PersistentParameters::clearParameters() {
    parameters.magic = 0xFF;
    parameters.size = 0xFF;
    memset(_pageBuffer, 0xFF, sizeof(_pageBuffer));
    write_parameters(FLASH_TARGET_OFFSET, _pageBuffer, sizeof(_pageBuffer));
}
