//
// Created by R on 7/13/23.
//

#ifndef PIFW_PERSISTENTPARAMETERS_H
#define PIFW_PERSISTENTPARAMETERS_H

#include <cstdint>
#include <hardware/flash.h>
#include <pico/sync.h>
#include <pico/platform.h>
#include "pi.pb.h"
#include "structures.h"

class PersistentParameters {
public:
    PersistentParameters();

    struct __attribute__((packed)) {
        uint32_t magic; // magic value to see if something resembling parameter struct is there
        uint32_t size;  // size of the struct, useful to void the parameters when a new one is added in firmware
        int8_t   ui_volume;
        int8_t ui_vibro_power;
        int8_t   ui_sensitivity;
        bool     ui_backlight_enabled;
        int8_t   ui_brightness;
        detection_mode_t ui_detection_mode;
        SamplingConfiguration md_sampling_configuration;
        ThresholdConfiguration md_threshold_configuration;
        bool     has_sampling_configuration;
        bool     has_threshold_configuration;
    } parameters;

    bool readParameters();

    void writeParameters();

    void clearParameters();

    critical_section_t criticalSection;
    /**
     * Implements disabling audio for the flash writes
     */
    volatile bool wantsFlashWrite;
    volatile bool canFlashWrite;
protected:
    uint8_t __attribute__((aligned (32))) _pageBuffer[FLASH_PAGE_SIZE];
    static const uint32_t FLASH_TARGET_OFFSET = PICO_FLASH_SIZE_BYTES - FLASH_SECTOR_SIZE;
    static const uint32_t EXPECTED_MAGIC = 0xDEADBEFF;
    static const uint32_t EXPECTED_SIZE = sizeof(parameters);
};


#endif //PIFW_PERSISTENTPARAMETERS_H
