//
// Created by R on 3/4/23.
//

#ifndef PIFW_PROTOCOLCOMM_H
#define PIFW_PROTOCOLCOMM_H

#include <pico/stdlib.h>
#include <hardware/uart.h>
#include "nanoparser.h"
#include "SampleAnalyzer.h"
#include "sampler/PISampler.h"
#include "ThresholdDetectorSM.h"
#include <pb_common.h>
#include "pi.pb.h"
#include <pico/util/queue.h>
#include "structures.h"
#include <StaticCircularBuffer.hpp>
#include "PersistentParameters.h"
#include "BatteryMonitor.h"
#include "Vibro.h"
#include "HAL.h"

class ProtocolComm {
public:
    ProtocolComm(PISampler &pi, ThresholdDetectorSM &td, HAL &hal);
    void log_message(const char* format, ...);

    void send_board_identification_message();
    void send_log_message(const char* msg, size_t len);

    void send_detection_measurement(const decay_sample_t& sample, SampleAnalyzer::sample_stats_t* stats=nullptr, float* amplitude=nullptr, float* phase=nullptr);

    void send_reference_measurement(const decay_sample_t& sample, SampleAnalyzer::sample_stats_t* stats=nullptr);

    void send_raw_sample(const RawSample& sample);

    void send_noise_floor_status_report();
    /**
     * Receive messages via USB and respond/execute received commands
     */
    void communicate();

    void init();
    
    typedef enum {
        PC_REQUEST_REFERENCE_SAMPLE_ACQUISITION=0,
        PC_REQUEST_AUTOMATIC_OFFSET_ADJUSTMENT,
        PC_REQUEST_NOISE_FLOOR_MEASUREMENT
    } protocol_command_t;

    typedef Queue<protocol_command_t, 4> ProtocolCommandQueue;

    ProtocolCommandQueue& getProtocolCommandQueue() { return _pcQueue; }

    void send_threshold_configuration();

    void onUartRXIRQ();

    bool send_system_status(const SystemStatus& status);

protected:
    size_t write_bytes(const uint8_t* data, size_t len);
    size_t read_bytes(uint8_t* data, size_t max_len);
    void process_received_message(parser_msg_t* msg);
    void copy_stats_to_measurement_sample(const SampleAnalyzer::sample_stats_t& stats, MeasurementSample& ms);
    uint8_t _outmsgbuf[512];
    uint8_t _inmsgbuf[128];
    parser_msg_t _parser_received_msg;
    PISampler& _pi;
    ThresholdDetectorSM&  _td;
    PersistentParameters& _params;
    BatteryMonitor&       _bm;
    Vibro &_vibro;
    ProtocolCommandQueue _pcQueue;
    void handle_sampling_configuration(SamplingConfiguration &cfg);

    void send_sampling_configuration();

    static const uint32_t DM_MAX_BANDWIDTH_BYTE_S = 10 * 1024; //max communications bandwidth reserved for detection measurements
    uint32_t _numDetectionMeasurements;


    void handle_threshold_configuration(const ThresholdConfiguration &conf);
    void handle_simulated_state(const SetSimulatedStateRequest& req);
    bool _uartInitialized;
    uart_inst_t* _uartInst;
    bool _initializeUART();
    Queue<uint8_t, 256> _uartRXBuffer;

};


#endif //PIFW_PROTOCOLCOMM_H
