//
// Created by R on 3/2/23.
//

#ifndef PIFW_SAMPLEANALYZER_H
#define PIFW_SAMPLEANALYZER_H


#include <cstdint>
#include "structures.h"
#include "StaticFIRFilter.hpp"

class SampleAnalyzer {
public:
    SampleAnalyzer();

    typedef struct {
        float min_amp;
        float max_amp;
        float mean_amp;
        float mean_amp_filtered;
        uint32_t clip_end_idx; // sample index at which clipping no longer occurs
        float lin_fit_slope;
        float lin_fit_offset;
        bool  lin_fit_valid;
        float exp_fit_exponent;
        float exp_fit_intercept;
        bool  exp_fit_valid;
        uint16_t pulse_len_us;
        float integral;
        float weighted_delta_ref;
        bool  top_clipping_detected;
    } sample_stats_t;

    static sample_stats_t sample_compute_stats(const decay_sample_t &sample, const decay_sample_t &ref_sample,
                                               uint32_t num_samples_to_analyze,
                                               bool compute_expensive_stats);
    static sample_stats_t
    sample_compute_stats(const decay_sample_t &sample, uint32_t num_samples_to_analyze, bool compute_expensive_stats);

    static bool sample_compute_linear_slope(uint32_t num_samples, const float *sample_buf, float& slope, float& offset);
    static bool sample_compute_exponential_decay_slope(uint32_t num_samples, const float *sample_buf, float& exponent, float& intercept);
    /**
     * Implement squared weighted delta ref (i.e. first samples give more weight to the output)
     * @param num_samples
     * @param sample_buf
     * @param ref_sample_buf
     * @return
     */
    static float sample_compute_weighted_delta_ref(uint32_t num_samples, const float* sample_buf, const float* ref_sample_buf);
    constexpr static const float    US_PER_SAMPLE=1.0f;
    constexpr static const float    CLIPPING_TOP_THRESHOLD = 16384.0f * 0.995f; // what value is considered to be clipping (within 0.5% of max)?
    constexpr static const float    CLIPPING_COUNT_THRESHOLD_PCT = 0.3f; //how many samples from the total need to be clipping for clipping detection to kick in?
protected:

};


#endif //PIFW_SAMPLEANALYZER_H
