//
// Created by R on 5/19/24.
//

#include "TemperatureMonitor.h"
#include <hardware/adc.h>

TemperatureMonitor::TemperatureMonitor(): _filtered_adc_value(0.0f) {

}

bool TemperatureMonitor::init() {
    adc_init();
    adc_set_temp_sensor_enabled(true);
    adc_select_input(ADC_TEMP_SENSOR_INPUT);
    _filtered_adc_value = adc_read();
    return true;
}

void TemperatureMonitor::update_temperature() {
    adc_select_input(ADC_TEMP_SENSOR_INPUT);
    const float beta = 0.95f;
    _filtered_adc_value = ((1.0f - beta) * (float)adc_read()) + (float)beta * _filtered_adc_value;
}

TemperatureMonitor::temperature_t TemperatureMonitor::read_temperature_C() {
    temperature_t t;
    t.mcu_temperature_adc_counts = (uint16_t)_filtered_adc_value;

    const float conversionFactor = 3.3f / (1 << 12);

    float adc = _filtered_adc_value * conversionFactor;
    t.mcu_temperature_C = 27.0f - (adc - 0.706f) / 0.001721f;
    return t;
}
