#include <hardware/adc.h>
#include <hardware/clocks.h>
#include <hardware/pwm.h>
#include "Vibro.h"
#include "util/math.h"

Vibro::Vibro() {
}

bool Vibro::init() {
    gpio_set_function(PIN_VIBRO, GPIO_FUNC_PWM);

    _vibro_pwm_slice = pwm_gpio_to_slice_num(PIN_VIBRO);
    _vibro_pwm_chan = pwm_gpio_to_channel(PIN_VIBRO);
    _vibro_pwm_period_ticks = clock_get_hz(clk_sys) / VIBRO_PWM_FREQ_HZ;

    pwm_set_enabled(_vibro_pwm_slice, true);
    pwm_set_wrap(_vibro_pwm_slice, _vibro_pwm_period_ticks);

    vibrate(0.0f);

    return true;
}

/**
 * Vibrate motor using PWM signal specified by the level power 0.0f to 1.0f
 * @return
 */
void Vibro::vibrate(float power) {
    if (!_have_simulated_vibro_pwm_dc) {
        power = map(power, 0.0f, 1.0f, 0.0f, _max_power);
        if (power < VIBRO_MIN_POWER) { // motor cant start with power less than 10% DC
            power = 0.0f;
        }
    } else {
        power = _simulated_vibro_pwm_dc;
    }
    set_vibro_pwm_dc(power);
}

void Vibro::set_simulated_vibro_pwm_duty_cycle(float pwm_dc) {
    _simulated_vibro_pwm_dc = pwm_dc;
    _have_simulated_vibro_pwm_dc = true;
    set_vibro_pwm_dc(pwm_dc);
}

void Vibro::clear_simulated_vibro_pwm_duty_cycle() {
    _have_simulated_vibro_pwm_dc = false;
    set_vibro_pwm_dc(0.0f);
}

void Vibro::set_vibro_pwm_dc(float pwm_dc) {
    _vibro_pwm_duty_cycle = pwm_dc;
    uint16_t pwm_level = (uint16_t) (_vibro_pwm_period_ticks - 1) * _vibro_pwm_duty_cycle;
    pwm_set_chan_level(_vibro_pwm_slice, _vibro_pwm_chan, pwm_level);
}
