#!/usr/bin/env python
import soundfile as sf
import samplerate
import os

def convert(infile, outfile, verbose=False):
    soundfile = infile
    data_in, datasamplerate = sf.read(soundfile)
    # This means stereo so extract one channel 0
    if len(data_in.shape) > 1:
        data_in = data_in[:, 0]
    converter = 'sinc_best'  # or 'sinc_fastest', ...
    desired_sample_rate = 11000.0
    ratio = desired_sample_rate / datasamplerate
    data_out = samplerate.resample(data_in, ratio, converter)
    maxValue = max(data_out)
    minValue = min(data_out)
    vrange = (maxValue - minValue)
    if verbose:
        print("Converting %s..." % infile)
        print("length=", len(data_out))
        print("max value=", max(data_out))
        print("min value=", min(data_out))
        print("value range=", vrange)
    varname = os.path.splitext(os.path.basename(infile))[0].upper()

    m68code = "#pragma once\n/*    File " + soundfile + "\n *    Sample rate " + str(int(desired_sample_rate)) + " Hz\n */\n"
    m68code += "#define WAV_DATA_LENGTH_%s " % varname + str(len(data_out)) + " \n\n"
    m68code += "const uint8_t WAV_DATA_%s[] = {\n    " % varname
    maxitemsperline = 16
    itemsonline = maxitemsperline
    firstvalue = 0
    lastvalue = 0
    for v in data_out:
        # scale v to between 0 and 1
        isin = (v - minValue) / vrange
        v = int((isin * 255))
        vstr = str(v)
        if (firstvalue == 0):
            firstvalue = v
        lastvalue = v
        m68code += vstr
        itemsonline -= 1
        if (itemsonline > 0):
            m68code += ','
        else:
            itemsonline = maxitemsperline
            m68code += ',\n    '

    # keep track of first and last values to avoid
    # blip when the loop restarts.. make the end value
    # the average of the first and last.
    end_value = int((firstvalue + lastvalue) / 2)
    m68code += str(end_value) + '    \n};'
    if outfile == "-":
        print(m68code)
    else:
        with open(outfile, "wb") as of:
            of.write(m68code.encode("utf8"))
    if verbose:
        print("%s converted" % infile)
    return 0


if __name__ == "__main__":
    import sys
    import os
    if len(sys.argv) <= 2:
        print("Usage: %s file.wav file.h [-d]" % os.path.basename(sys.argv[0]))
        sys.exit(1)

    v = False
    if len(sys.argv) >= 4 and sys.argv[3] == "-d":
        v = True
    sys.exit(convert(sys.argv[1], sys.argv[2], v))
