//
// Created by R on 4/17/23.
//

#ifndef PIFW_ADC_H
#define PIFW_ADC_H

#include <pico/stdlib.h>
#include <hardware/spi.h>
#include <hardware/dma.h>
#include <hardware/pio.h>

class ADC {
public:
    ADC();
    void init(PIO pio, uint32_t spi_reader_sm);

    uint32_t acquire_sample_blocking();
    void acquire_samples_dma_blocking(uint32_t* buf, size_t num_samples);
    void acquire_samples_dma_start(uint32_t* buf, size_t num_samples, bool shift_samples_in_irq=true) {
            if (shift_samples_in_irq) {
                _dmaBuf = buf;
                _dmaLen = num_samples;
            } else {
                _dmaBuf = nullptr;
            }
            dma_channel_set_write_addr(_dma_chan, buf, false);
            pio_sm_clear_fifos(_spi_pio, _spi_sm);
            dma_channel_set_trans_count(_dma_chan, num_samples, true);
    }

    bool dma_transaction_finished();
    void shift_dma_samples(uint32_t* buf, size_t num_samples);
    typedef void (*adc_callback_t)();
    void register_dma_finished_callback(adc_callback_t fn);

    void execute_dma_finished_callback();
protected:
    PIO             _spi_pio;
    uint            _spi_sm;
    int            _dma_chan;
    void           _init_dma();
    static const uint32_t _spi_freq_hz = 24000000;
    static const uint32_t _gpio_sclk = 13;
    static const uint32_t _gpio_sdi = 12;
    static const uint32_t _gpio_cs = _gpio_sclk + 1; // fixed by PIO program design

    adc_callback_t _dmaFinishedCallback;
    uint32_t*      _dmaBuf;
    size_t         _dmaLen;
};


#endif //PIFW_ADC_H
