#include <pico/stdlib.h>
#include <pico/multicore.h>
#include <tusb.h>
#include <pico/stdio/driver.h>
#include "config.h"
#include "sampler/PISampler.h"
#include "UI.h"
#include "SampleAnalyzer.h"
#include "ProtocolComm.h"
#include <pico/util/queue.h>
#include "ThresholdDetectorSM.h"
#include "IndicationStateMachine.h"
#include "PIMD.h"
#include <pico/multicore.h>
#include "audio/audio.h"
#include "HAL.h"

HAL hal;
PISampler piSampler(hal.adc);
UI md_ui(hal);
IndicationStateMachine ism(hal, md_ui);
ThresholdDetectorSM td;
ProtocolComm pc(piSampler, td, hal);
PIMD pimd(piSampler, pc, md_ui, ism, td, hal);


/**
 * For now the multicore code is implemented in the following way:
 * core0 handles everything
 * core1 handles audio.
 *
 */

void core1_entry() {
    //multicore_lockout is used to make sure this core is not accessing flash while parameters are being written
    multicore_lockout_victim_init();
    hal.audio.init();
    uint64_t next_step_timestamp = time_us_64() + 1000;
    while (true) {
        audio_playback_step();
        auto t = time_us_64();
        if (t > next_step_timestamp) {
            hal.audio.step();
            next_step_timestamp = t + 1000;
        }
    }
}

int main() {
    set_sys_clock_khz(176000, true);
    static const uint32_t PIN_POWER_EN=7;
    gpio_init(PIN_POWER_EN);
    gpio_set_dir(PIN_POWER_EN, true);
    gpio_put(PIN_POWER_EN, true);

    //sleep so that any open serial terminals die
    tud_disconnect();
    multicore_reset_core1();
    multicore_launch_core1(core1_entry);
    //audio initializtion happens on core 1
    while (!hal.audio.isInitialized())
        sleep_ms(1);

    hal.audio.enableOutputDClass();
    stdio_init_all();
    pc.init();
    md_ui.init();
    hal.vibro.init();

    //check if actually need to start up
    md_ui.startup_loop();

    hal.bm.init();
    hal.tm.init();
    md_ui.led_animation_blocking();
    piSampler.init();
    //md_pi.setRawSampleAcquisition(true);
    ism.init();

    //update the battery state before announcing it
    for (int i=0; i < 100; i++) {
        hal.bm.update_battery_charge();
        sleep_ms(1);
    }

    while (1) {
        pimd.step();
    }
}
