#!/usr/bin/env python
import logging
import piapi
import sys
import numpy as np
import pi_pb2
import time
import datetime
import argparse
logging.TRACE = 9
logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s: %(message)s")

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("port")
    args = parser.parse_args()
    api = piapi.PIAPI(port=args.port)
    log = logging.getLogger("main")
    next_status_print_time = time.time() + 5
    msg_received_num = {
        "DetectionMeasurement": 0,
        "SystemStatus": 0
    }
    sample_cols = ("mean_amp", "integral", "weighted_delta_ref", "mean_amp_filtered")
    print(f"timestamp_us,{",".join(sample_cols)},tempC")
    current_temp_C = None
    start_timestamp_us = None
    try:
        while True:
            msgs = api.receive_messages(timeout=0.01, abort_early=False)
            for m in msgs:
                if type(m) == pi_pb2.DetectionMeasurement:
                    msg_received_num["DetectionMeasurement"] += 1
                    if start_timestamp_us is None:
                        start_timestamp_us = m.timestamp_us
                    current_sample_stats = [str(getattr(m.sample, col)) for col in sample_cols]
                    if current_temp_C is not None:
                        ts = m.timestamp_us - start_timestamp_us
                        print(f"{ts},{",".join(current_sample_stats)},{current_temp_C}")
                elif type(m) == pi_pb2.SystemStatus:
                    msg_received_num["SystemStatus"] += 1
                    current_temp_C = m.mcu_temperature_c

            if time.time() > next_status_print_time:
                next_status_print_time = time.time() + 1
                logging.info(f"Messages received: {msg_received_num}")

    except KeyboardInterrupt:
        pass

if __name__ == "__main__":
    main()

