//
// Created by R on 23.18.2.
//

#ifndef PIFW_PI_H
#define PIFW_PI_H

#include <sys/types.h>
#include <cstdint>

#include "../SampleAnalyzer.h"
#include "../hal/ADC.h"
#include "pi.pb.h"
#include <pico/util/queue.h>
#include <StaticCircularBuffer.hpp>
#include <hardware/pwm.h>
#include "../util/Queue.h"
#include "../structures.h"
#include <cmath>

class PISampler {
public:
    PISampler(ADC& adc);
    bool init();

    /**
     * Sets offset PWM duty cycle (0..1)
     * @param dc duty cycle between 0 and 1
     */
    void set_offset_duty_cycle(float dc, bool manual=false);

    float get_offset_duty_cycle() const { return _frontend_offset_duty_cycle; }

    static const uint32_t ADC_SAMPLE_DEPTH = MAX_DECAY_SAMPLES;

    inline DecaySampleQueue& getDecaySampleQueue() { return decaySampleQueue; }

    void clearDecaySampleHistory() { _sampleHistoryBufClearReq = true;}

    typedef struct {
        uint32_t sample[ADC_SAMPLE_DEPTH];
    } raw_sample_item_t;

    const SamplingConfiguration& get_sampling_configuration() const {
        return _sampling_cfg;
    }

    bool set_sampling_configuration(const SamplingConfiguration& cfg);

    inline queue_t* getRawSamplesQueue() { return &_rawSamplesQueue; }

    void setRawSampleAcquisition(bool acq) { _acquireRawSamples = acq; }

    uint32_t get_sample_frequency_hz() const { return 1000000 / _sampling_cfg.pulse_period_us; }

    void setupPIOSampling();

    /**
     *  Checks and updates the sampling configuration (pulse length and period) to be suitable for
     *  PIO sampling
     */
    void checkSamplingConfigurationForPIO();

    void adcDMAFinishedCb();

    bool samplingConfigurationChanged();

    uint32_t getNumADCDecayCurvesRead() const { return _numAdcDecayCurvesRead; }
    void     setNumADCDecayCurvesRead(uint32_t n) { _numAdcDecayCurvesRead = n; }


protected:
    ADC&    _adc;
    bool    _pwm_init();
    /**
     * Sets the control GPIOs to set the gain level
     * @param gain
     * @return
     */
    bool    _set_frontend_gain(const SamplingConfiguration_FrontendGainLevel& gain);

    uint32_t _offset_pwm_slice;
    uint32_t _offset_pwm_chan;
    uint32_t _offset_pwm_period_ticks;
    PIO      _pio;
    uint32_t _pulse_period_sm;
    uint32_t _pulse_length_sm;
    void updatePIOSamplingConfiguration();

    static const uint32_t OFFSET_PWM_FREQ_HZ = 500 * 1000;
    static const uint32_t PIN_TX=25;
    static const uint32_t PIN_OFFSET_PWM=22;
    static const uint32_t PIN_GAIN_10K = 23;
    static const uint32_t PIN_GAIN_4_22K = 24;

    static const uint32_t PIN_RX=26;
    static const uint32_t RX_ADC_NUM = PIN_RX - 26;
    uint32_t _sample_buf[ADC_SAMPLE_DEPTH];

    SamplingConfiguration _sampling_cfg;

    /**
     * Oversampled decay @ sampling frequency
     */
    DecaySampleQueue decaySampleQueue;

    uint32_t _sampleCounter;

    queue_t _rawSamplesQueue;

    bool    _acquireRawSamples;

    /** IRQ variables **/
    uint    _txPWMSlice;
    uint    _txPWMChan;
    raw_sample_item_t _currentADCSample;
    static const uint32_t MAX_SAMPLE_HISTORY=256;
    StaticCircularBuffer<raw_sample_item_t, MAX_SAMPLE_HISTORY> _sampleHistoryBuf;

    void _debugInit();
    bool _debugEnabled;
    static const uint32_t DBG1_PIN = 2;
    static const uint32_t DBG2_PIN = 3;
    float _frontend_offset_duty_cycle;
    bool _sampleHistoryBufClearReq;
    bool _samplingConfigurationChanged;
    volatile uint32_t _numAdcDecayCurvesRead;

};


#endif //PIFW_PI_H
