//
// Created by R on 9/25/23.
//

#ifndef PIFW_QUEUE_H
#define PIFW_QUEUE_H

#include <pico/util/queue.h>
/**
 * C++ wrapper around pico/util/queue.
 *
 * Usage:
 *
 * Queue<mystruct, 5> q; // creates a queue for 5 elements of mystruct
 *
 * @tparam T
 * @tparam TElemCount
 */
template<class T, size_t TElemCount>
class Queue {
public:
    Queue() {
        queue_init(&q, sizeof(T), TElemCount);
    }

    Queue(const Queue& oq) {
        this->q = oq.q;
    }

    ~Queue() {
        queue_free(&q);
    }

    inline bool is_empty() { return queue_is_empty(&q); }
    inline bool is_full() { return queue_is_full(&q); }
    inline uint32_t get_level() { return queue_get_level(&q); }


    inline void add_blocking(const T* elem) { queue_add_blocking(&q, elem); }
    inline void add_blocking(const T& elem) { queue_add_blocking(&q, &elem); }
    inline bool try_add(const T* elem) { return queue_try_add(&q, elem); }
    inline bool try_add(const T& elem) { return queue_try_add(&q, &elem); }

    inline T    remove_blocking() { T elem; queue_remove_blocking(&q, &elem); return elem; }
    inline bool try_remove(T* elem) { return queue_try_remove(&q, elem);}
    inline bool try_remove(T& elem) { return queue_try_remove(&q, &elem);}

protected:
    queue_t q;
};


#endif //PIFW_QUEUE_H
